grammar SCSPTree;

options
{
   output=AST;
   ASTLabelType=CommonTree;
   memoize=true;
   language=CSharp2; 
}

tokens {
SPEC_NODE;
PROCESS_NODE;
PROC_DEF_NODE;
DEADLOCK_NODE;
REACH_NODE;
}

@namespace	{PAT.SCSP.LTS}

@header{
using System;
using PAT.Common;
}

@members
{

   public Stack<String> paraphrases = new Stack<String>(); 
   public override String GetErrorMessage(RecognitionException e, String[] tokenNames)
    {
    	string msg = null;
    	if ( e is NoViableAltException ) {    		
		msg = "Invalid Symbol: '" + e.Token.Text + "'";
    	}
    	else 
    	{
    		msg = base.GetErrorMessage(e, tokenNames);    		
    	}

        if (paraphrases.Count > 0)
        {
            String paraphrase = (String)paraphrases.Peek();
            msg = msg + " " + paraphrase;
        }
        
        msg = msg + "\n" + "Line " + e.Token.Line + ", Column " + e.Token.CharPositionInLine;
        
    	return msg;             
    }

}

@rulecatch {
catch (RecognitionException re) 
{		
     String sError=GetErrorMessage(re, tokenNames); 
     throw new ParsingException(sError, re.Token);
}
}

public
specDef	
@init  { paraphrases.Push(" SCSP input analysis");  }
@after { paraphrases.Pop(); }
:	
	((p+=process)+ (a+=assertion)+)
	-> ^(SPEC_NODE $p+ $a+)
;

process
@init  { paraphrases.Push(" process equation analysis");  }
@after { paraphrases.Pop(); }
	:	
	v=VAR '=' p=processDef ';'
	-> ^(PROCESS_NODE $v $p)
	;
	
processDef
@init  { paraphrases.Push(" process definition analysis");  }
@after { paraphrases.Pop(); }
	:
	 v=VAR '->' p=processDef
	  -> ^(PROC_DEF_NODE $v $p)
	| v1=VAR '|||' v2=VAR
	  -> ^(PROC_DEF_NODE $v1 $v2)
	| s=SKIP
	  -> ^(PROC_DEF_NODE $s)
	| v3=VAR
	  -> ^(PROC_DEF_NODE $v3)
	;

assertion
	:	
	  '#assert' v=VAR 'deadlockfree'';'
	  -> ^(DEADLOCK_NODE $v)
	| '#assert' v1=VAR 'reaches' v2=VAR ';'
	  -> ^(REACH_NODE $v1 $v2)
	;

SKIP	:	
	'Skip'
;	
	
VAR : ('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'0'..'9'|'_')* 
	;

COMMENT
    :   '/*' ( options {greedy=false;} : . )* '*/' {$channel=Hidden;}
    ;
    
LINE_COMMENT
    : '//' ~('\n'|'\r')* '\r'? '\n' {$channel=Hidden;}
;

WS  :  (' ' | '\t' | '\n' | '\r' | '\f')+ {$channel = Hidden;};
